==> nui_mini_program-0.0.33.tar.gz: lib/util/nui_miniprogram_file_manager.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';
import 'package:archive/archive.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_database/models/enums/nui_database_query_type.dart';
import 'package:nui_mini_program/database/nui_miniprogram_database.dart';
import 'package:nui_mini_program/model/nui_miniprogram_module.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';
import 'dart:io';
import 'package:http/http.dart' as http;
import 'dart:math' as math;
import 'package:path/path.dart' as path;
import 'dart:ui' as ui;


// Override this in asset_bundle.dart file to overcome the Unable to find asset issue
// class PlatformAssetBundle extends CachingAssetBundle {
//   @override
//   Future<ByteData> load(String key) async {
//     print("AssetBundle Load asset for key: $key");
//     final Uint8List encoded = utf8.encoder.convert(Uri(path: Uri.encodeFull(key)).path);
//     final ByteData? backupAsset = await NUIMiniProgramFileManager.loadAsset(key);
//     final ByteData? asset =
//         backupAsset ?? await ServicesBinding.instance!.defaultBinaryMessenger.send('flutter/assets', encoded.buffer.asByteData());
//     if (asset == null) {
//       throw FlutterError('Unable to load asset: $key');
//     }
//     return asset;
//   }
// }

class NUIMiniProgramFileManager{
  NUIMiniProgramFileManager(NUIMiniProgramModule module){
    _module = module;
  }

  static const MODULE = "NUIMiniProgramFileManager";
  static const _FLUTTER_ASSETS_CHANNEL = "flutter/assets";

  NUIMiniProgramModule _module;

  Future<String> initializeMiniProgram() async{
    final versionControl = await _checkModuleVersion();
    print("NUIMiniProgramFileManager: Finished version control with result : $versionControl");
    if(versionControl){
      _updateRecordInDB();
      final indexHtml = await getMiniProgramIndexHtml();
      final file = File(indexHtml);
      final exist = await file.exists();
      print("NUIMiniProgramFileManager: Html=$indexHtml, exists=$exist");
      return indexHtml;
    }
    return null;
  }

  Future<String> getMiniProgramIndexHtml() async{
    final directoryPath = await _getArchiveDirectoryPath(_module.id);
    return "$directoryPath/index.html";
  }

  Future<Uint8List> getMiniProgramLoaderJson() async{
    final directoryPath = await _getArchiveDirectoryPath(_module.id);
    final filePath = "$directoryPath/app_loader.json";
    final file = File(filePath);

    if(!file.existsSync()) return null;

    final bytes = await file.readAsBytes();
    return bytes;
  }

  Future<bool> _updateRecordInDB() async{
    _module.createdAt = _module.createdAt ?? DateTime.now();
    _module.updatedAt = DateTime.now();
    final result = await NUIMiniProgramDatabase.get().save(_module);
    return result;
  }

  Future<bool> hasMiniProgram() async{
    final prevVersion = await NUIMiniProgramDatabase.get().getOne(NUIMiniProgramModuleEnt(), condition: (query){
      return query.where(field: "id", type: NUIEntType.STRING, condition: NUIDBQueryType.EQUALS, value: _module.id);
    });
    return prevVersion != null;
  }

  Future<bool> hasMiniProgramUpdate() async{
    return _hasVersionUpdate();
  }

  Future<bool> _hasVersionUpdate() async{
    final prevVersion = await NUIMiniProgramDatabase.get().getOne(NUIMiniProgramModuleEnt(), condition: (query){
      return query.where(field: "id", type: NUIEntType.STRING, condition: NUIDBQueryType.EQUALS, value: _module.id);
    });

    if(prevVersion == null){
      return true;
    }
    else{
      final curModuleVersion = _module.version;
      final prevModuleVersion = prevVersion.version;

      final diff = _compareNewAndOldVersion(curModuleVersion, prevModuleVersion);
      if(diff > 0 || diff < 0){
        return true;
      }
      else{
        //No changes found, so no action needed
        return false;
      }
    }
  }

  Future<bool> _checkModuleVersion() async{
    try{
      final hasVersionChange = await _hasVersionUpdate();
      if(hasVersionChange){
        final zipFile = File(await _getZipFileDownloadName(_module.id));
        final folder = File(await _getArchiveDirectoryPath(_module.id));

        if(zipFile.existsSync()){
          zipFile.deleteSync(recursive: true);
        }
        if(folder.existsSync()){
          folder.deleteSync(recursive: true);
        }

        final sync = await _downloadZipFromNetwork(_module.sourceUrl, _module.id);
        return sync != null;
      }
      return true;

    }catch(e, s){
      print("[NUIMiniProgramFileManager] Failed to check module version with error: $e, stacktrace: $s");
    }
    return false;
  }

  Future<String> getPreviousModuleVersion() async{
    final prevVersion = await NUIMiniProgramDatabase.get().getOne(NUIMiniProgramModuleEnt(), condition: (query){
      return query.where(field: "id", type: NUIEntType.STRING, condition: NUIDBQueryType.EQUALS, value: _module.id);
    });
    return prevVersion?.version;
  }

  int _compareNewAndOldVersion(String newVersion, String oldVersion){
    final newSplitted = newVersion.split(".");
    final oldSplitted = oldVersion.split(".");

    int difference;
    for(int i = 0; i < math.max(newSplitted.length, oldSplitted.length); i ++){
      final newInt = int.parse(newSplitted.getOrNull(i) ?? "0");
      final oldInt = int.parse(oldSplitted.getOrNull(i) ?? "0");

      if(newInt != oldInt){
        //A difference found
        difference = newInt - oldInt;
        break;
      }
    }

    //Positive indicates that a newer version is available, negative indicates an older version is used, zero indicates no version change
    return difference ?? 0;
  }

  Future<bool> requestPermissions(){
    return NUIPermissionUtil.requestPermissions([Permission.storage]);
  }

  Future<ByteData> _sendPlatformMessage(String channel, message) async{
    final Completer<ByteData> completer = Completer<ByteData>();
    ui.window.sendPlatformMessage("flutter/assets", message, (ByteData reply) {
      try {
        completer.complete(reply);
      } catch (exception, stack) {
        FlutterError.reportError(FlutterErrorDetails(
          exception: exception,
          stack: stack,
          library: 'services library',
          context: ErrorDescription('during a platform message response callback'),
        ));
      }
    });
    return completer.future;
  }

  static Future<ByteData> loadAsset(String key) async{
    final String assetFolderPath = Platform.environment['UNIT_TEST_ASSETS'];
    final String prefix =  'packages/${Platform.environment['APP_NAME']}/';
    final String channel = _FLUTTER_ASSETS_CHANNEL;

    File asset = isNullOrEmpty(assetFolderPath) ? File(key) : File(path.join(assetFolderPath, key));

    if (!asset.existsSync()) {
      if (!key.startsWith(prefix)) {
        return null;
      }

      key = key.replaceFirst(prefix, '');
      asset = !isNullOrEmpty(assetFolderPath) ? File(path.join(assetFolderPath, key)) : File(key);
      if (!asset.existsSync()) {
        return null;
      }
    }

    final Uint8List encoded = Uint8List.fromList(asset.readAsBytesSync());
    return Future<ByteData>.value(encoded.buffer.asByteData());
  }

  //Extracted from Flutter _binding_io.dart and binding.dart for minor tweak on assets handling
  void enableDynamicAssets(String filePath){
    print("Adding asset channel for mini program: ${_module.id}");
    NUISystem.setAssetChannel(_module.id, (key) async {
      return loadAsset(key);
    });
    // print("NUIMiniProgramFileManager, Enabling dynamic assets for $filePath");
    // final String assetFolderPath = Platform.environment['UNIT_TEST_ASSETS'];
    // final String prefix =  'packages/${Platform.environment['APP_NAME']}/';
    // final String channel = _FLUTTER_ASSETS_CHANNEL;
    //
    //
    // ServicesBinding.instance.defaultBinaryMessenger.setMessageHandler(channel, (ByteData message) {
    //   print("NUIMiniProgramFileManager: Received message handler for flutter assets");
    //   String key = utf8.decode(message.buffer.asUint8List());
    //   File asset = isNullOrEmpty(assetFolderPath) ? File(key) : File(path.join(assetFolderPath, key));
    //
    //   if (!asset.existsSync()) {
    //     if (!key.startsWith(prefix)) {
    //       return _sendPlatformMessage(channel, message);
    //     }
    //
    //     key = key.replaceFirst(prefix, '');
    //     asset = !isNullOrEmpty(assetFolderPath) ? File(path.join(assetFolderPath, key)) : File(key);
    //     if (!asset.existsSync()) {
    //       return _sendPlatformMessage(channel, message);
    //     }
    //   }
    //
    //   final Uint8List encoded = Uint8List.fromList(asset.readAsBytesSync());
    //   return Future<ByteData>.value(encoded.buffer.asByteData());
    // });
  }

  void disableDynamicAssets(String path){
    print("Removing asset channel mini program: ${_module.id}");
    NUISystem.removeAssetChannel(_module.id);
    // ServicesBinding.instance.defaultBinaryMessenger.setMockMessageHandler(_FLUTTER_ASSETS_CHANNEL, null);
  }

  Future<Directory> _getExternalStorageDirectory({bool isExternal: false}){
    if(isExternal == true) {
      return getExternalStorageDirectory();
    }
    return getApplicationDocumentsDirectory();
  }

  Future<File> _downloadFileFromNetwork(String url, String module, {String extension: "zip"}) async {
    if(!(await requestPermissions())){
      logNUI(MODULE, "Please allow storage permission for the app for downloading file");
      return null;
    }

    try{
      var req = await http.Client().get(Uri.parse(url));
      final bytes = req.bodyBytes;
      logNUI(MODULE, "Downloaded the zip file of size : ${bytes.length}");
      final downloadPath = await _getZipFileDownloadName(module, extension: extension);
      var file = File(downloadPath);
      return file.writeAsBytes(bytes);
    }catch(e, s){
      logNUI(MODULE, "Failed to download file from network path: $url with error : $e, stack : $s");
      return null;
    }
  }

  String _getZipFileNameByModule(String module, {String extension: "zip"}){
    return "$module.${extension ?? "zip"}";
  }

  Future<String> _getZipFileDownloadName(String module, {String extension: "zip"}) async{
    final directory = await _getExternalStorageDirectory();
    return "${directory.path}/${_getZipFileNameByModule(module, extension: extension)}";
  }

  Future<File> _downloadZipFromNetwork(String zipUrl, String module, {String extension: "zip"}) async {
    if(!(await requestPermissions())){
      logNUI(MODULE, "Please allow storage permission for the app for downloading zip");
      return null;
    }

    logNUI(MODULE, "Starting download zip file from url : $zipUrl for module : $module ...");
    final zippedFile = await _downloadFileFromNetwork(zipUrl, module, extension: extension);
    if(zippedFile == null){
      logNUI(MODULE, "Failed to download zip file for module : $module");
      return null;
    }
    else {
      logNUI(MODULE, "Download zip file for module : $module is successful, starting unzip ...");
      final unzipped = await _unarchiveAndSave(zippedFile, module);

      if(unzipped){
        logNUI(MODULE, "Zip file download and unzip completed for module : $module");
        return zippedFile;
      }
      else{
        logNUI(MODULE, "Zip file downloaded but unzip failed, deleting zip of module : $module ...");
        zippedFile.delete();
        logNUI(MODULE, "Zip file deleted for module : $module");
        return null;
      }
    }
  }

  Future<String> _getArchiveDirectoryPath(String module) async{
    final directory = await _getExternalStorageDirectory();
    final directoryPath = "${directory.path}/$module";
    return directoryPath;
  }

  // Unarchive and save the file in Documents directory and save the paths in the array
  Future<bool> _unarchiveAndSave(File zippedFile, String module) async {
    if(!(await requestPermissions())){
      logNUI(MODULE, "Please allow storage permission for the app for unzip");
      return false;
    }

    final directoryPath = await _getArchiveDirectoryPath(module);

    logNUI(MODULE, "Start unzipping zip file into path : $directoryPath");
    String parentFolder;
    try{
      var bytes = zippedFile.readAsBytesSync();
      var archive = ZipDecoder().decodeBytes(bytes);
      for (var file in archive) {
        logNUI(MODULE, "Original file name: ${file.name}");
        final zipFileName = parentFolder != null ? file.name.replaceFirst("$parentFolder/", "") : file.name;
        var fileName = '$directoryPath/$zipFileName';
        if (file.isFile) {
          logNUI(MODULE, "Inner file in zip : $fileName");
          var outFile = File(fileName);
          outFile = await outFile.create(recursive: true);
          await outFile.writeAsBytes(file.content);
        }
        else{
          //The folder names
          final splittedFileName = file.name.split("/"); //To remove the folder name of the zip file
          splittedFileName.remove("");
          if(splittedFileName.length == 1){
            //The parent folder
            parentFolder = splittedFileName[0];
            logNUI(MODULE, "The parent folder name : $parentFolder");
          }
        }
      }
      logNUI(MODULE, "Unzip zip files successful with ${archive.length} files");
      return true;
    }catch(e){
      logNUI(MODULE, "Unzip files failed with error : $e");
      return false;
    }
  }

}

==> nui_mini_program-1.0.5.tar.gz: lib/util/nui_miniprogram_file_manager.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';
import 'package:archive/archive.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_database/models/enums/nui_database_query_type.dart';
import 'package:nui_mini_program/database/nui_miniprogram_database.dart';
import 'package:nui_mini_program/model/nui_miniprogram_module.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';
import 'dart:io';
import 'package:http/http.dart' as http;
import 'dart:math' as math;
import 'package:path/path.dart' as path;
import 'dart:ui' as ui;


// Override this in asset_bundle.dart file to overcome the Unable to find asset issue
// class PlatformAssetBundle extends CachingAssetBundle {
//   @override
//   Future<ByteData> load(String key) async {
//     print("AssetBundle Load asset for key: $key");
//     final Uint8List encoded = utf8.encoder.convert(Uri(path: Uri.encodeFull(key)).path);
//     final ByteData? backupAsset = await NUIMiniProgramFileManager.loadAsset(key);
//     final ByteData? asset =
//         backupAsset ?? await ServicesBinding.instance!.defaultBinaryMessenger.send('flutter/assets', encoded.buffer.asByteData());
//     if (asset == null) {
//       throw FlutterError('Unable to load asset: $key');
//     }
//     return asset;
//   }
// }

class NUIMiniProgramFileManager{
  NUIMiniProgramFileManager(NUIMiniProgramModule module){
    _module = module;
  }

  static const MODULE = "NUIMiniProgramFileManager";
  static const _FLUTTER_ASSETS_CHANNEL = "flutter/assets";

  late NUIMiniProgramModule _module;

  Future<String> initializeMiniProgram() async{
    final versionControl = await _checkModuleVersion();
    print("NUIMiniProgramFileManager: Finished version control with result : $versionControl");
    if(versionControl){
      _updateRecordInDB();
      final indexHtml = await getMiniProgramIndexHtml();
      final file = File(indexHtml);
      final exist = await file.exists();
      print("NUIMiniProgramFileManager: Html=$indexHtml, exists=$exist");
      return indexHtml;
    }
    return '';
  }

  Future<String> getMiniProgramIndexHtml() async{
    final directoryPath = await _getArchiveDirectoryPath(_module.id);
    return "$directoryPath/index.html";
  }

  Future<Uint8List?> getMiniProgramLoaderJson() async{
    final directoryPath = await _getArchiveDirectoryPath(_module.id);
    final filePath = "$directoryPath/app_loader.json";
    final file = File(filePath);

    if(!file.existsSync()) return null;

    final bytes = await file.readAsBytes();
    return bytes;
  }

  Future<bool> _updateRecordInDB() async{
    _module.createdAt = _module.createdAt ?? DateTime.now();
    _module.updatedAt = DateTime.now();
    final result = await NUIMiniProgramDatabase.get().save(_module);
    return result;
  }

  Future<bool> hasMiniProgram() async{
    final prevVersion = await NUIMiniProgramDatabase.get().getOne(NUIMiniProgramModuleEnt(), condition: (query){
      return query.where(field: "id", type: NUIEntType.STRING, condition: NUIDBQueryType.EQUALS, value: _module.id);
    });
    return prevVersion != null;
  }

  Future<bool> hasMiniProgramUpdate() async{
    return _hasVersionUpdate();
  }

  Future<bool> _hasVersionUpdate() async{
    final prevVersion = await NUIMiniProgramDatabase.get().getOne(NUIMiniProgramModuleEnt(), condition: (query){
      return query.where(field: "id", type: NUIEntType.STRING, condition: NUIDBQueryType.EQUALS, value: _module.id);
    });

    if(prevVersion == null){
      return true;
    }
    else{
      final curModuleVersion = _module.version;
      final prevModuleVersion = prevVersion.version;

      final diff = _compareNewAndOldVersion(curModuleVersion, prevModuleVersion);
      if(diff > 0 || diff < 0){
        return true;
      }
      else{
        //No changes found, so no action needed
        return false;
      }
    }
  }

  Future<bool> _checkModuleVersion() async{
    try{
      final hasVersionChange = await _hasVersionUpdate();
      if(hasVersionChange){
        final zipFile = File(await _getZipFileDownloadName(_module.id));
        final folder = File(await _getArchiveDirectoryPath(_module.id));

        if(zipFile.existsSync()){
          zipFile.deleteSync(recursive: true);
        }
        if(folder.existsSync()){
          folder.deleteSync(recursive: true);
        }

        final sync = await _downloadZipFromNetwork(_module.sourceUrl ?? '', _module.id);
        return sync != null;
      }
      return true;

    }catch(e, s){
      print("[NUIMiniProgramFileManager] Failed to check module version with error: $e, stacktrace: $s");
    }
    return false;
  }

  Future<String?> getPreviousModuleVersion() async{
    final prevVersion = await NUIMiniProgramDatabase.get().getOne(NUIMiniProgramModuleEnt(), condition: (query){
      return query.where(field: "id", type: NUIEntType.STRING, condition: NUIDBQueryType.EQUALS, value: _module.id);
    });
    return prevVersion.version;
  }

  int _compareNewAndOldVersion(String newVersion, String oldVersion){
    final newSplitted = newVersion.split(".");
    final oldSplitted = oldVersion.split(".");

    int? difference;
    for(int i = 0; i < math.max(newSplitted.length, oldSplitted.length); i ++){
      final newInt = int.parse(newSplitted.getOrNull(i) ?? "0");
      final oldInt = int.parse(oldSplitted.getOrNull(i) ?? "0");

      if(newInt != oldInt){
        //A difference found
        difference = newInt - oldInt;
        break;
      }
    }

    //Positive indicates that a newer version is available, negative indicates an older version is used, zero indicates no version change
    return difference ?? 0;
  }

  Future<bool> requestPermissions(){
    return NUIPermissionUtil.requestPermissions([Permission.storage]);
  }

  Future<ByteData?> _sendPlatformMessage(String channel, message) async{
    final Completer<ByteData> completer = Completer<ByteData>();
    ui.window.sendPlatformMessage("flutter/assets", message, (ByteData? reply) {
      try {
        completer.complete(reply);
      } catch (exception, stack) {
        FlutterError.reportError(FlutterErrorDetails(
          exception: exception,
          stack: stack,
          library: 'services library',
          context: ErrorDescription('during a platform message response callback'),
        ));
      }
    });
    return completer.future;
  }

  static Future<ByteData?> loadAsset(String key) async{
    final String assetFolderPath = Platform.environment['UNIT_TEST_ASSETS']!;
    final String prefix =  'packages/${Platform.environment['APP_NAME']}/';
    final String channel = _FLUTTER_ASSETS_CHANNEL;

    File asset = isNullOrEmpty(assetFolderPath) ? File(key) : File(path.join(assetFolderPath, key));

    if (!asset.existsSync()) {
      if (!key.startsWith(prefix)) {
        return null;
      }

      key = key.replaceFirst(prefix, '');
      asset = !isNullOrEmpty(assetFolderPath) ? File(path.join(assetFolderPath, key)) : File(key);
      if (!asset.existsSync()) {
        return null;
      }
    }

    final Uint8List encoded = Uint8List.fromList(asset.readAsBytesSync());
    return Future<ByteData>.value(encoded.buffer.asByteData());
  }

  //Extracted from Flutter _binding_io.dart and binding.dart for minor tweak on assets handling
  void enableDynamicAssets(String filePath){
    print("Adding asset channel for mini program: ${_module.id}");
    NUISystem.setAssetChannel(_module.id, (key) async {
      return (await loadAsset(key))!;
    });
    // print("NUIMiniProgramFileManager, Enabling dynamic assets for $filePath");
    // final String assetFolderPath = Platform.environment['UNIT_TEST_ASSETS'];
    // final String prefix =  'packages/${Platform.environment['APP_NAME']}/';
    // final String channel = _FLUTTER_ASSETS_CHANNEL;
    //
    //
    // ServicesBinding.instance.defaultBinaryMessenger.setMessageHandler(channel, (ByteData message) {
    //   print("NUIMiniProgramFileManager: Received message handler for flutter assets");
    //   String key = utf8.decode(message.buffer.asUint8List());
    //   File asset = isNullOrEmpty(assetFolderPath) ? File(key) : File(path.join(assetFolderPath, key));
    //
    //   if (!asset.existsSync()) {
    //     if (!key.startsWith(prefix)) {
    //       return _sendPlatformMessage(channel, message);
    //     }
    //
    //     key = key.replaceFirst(prefix, '');
    //     asset = !isNullOrEmpty(assetFolderPath) ? File(path.join(assetFolderPath, key)) : File(key);
    //     if (!asset.existsSync()) {
    //       return _sendPlatformMessage(channel, message);
    //     }
    //   }
    //
    //   final Uint8List encoded = Uint8List.fromList(asset.readAsBytesSync());
    //   return Future<ByteData>.value(encoded.buffer.asByteData());
    // });
  }

  void disableDynamicAssets(String path){
    print("Removing asset channel mini program: ${_module.id}");
    NUISystem.removeAssetChannel(_module.id);
    // ServicesBinding.instance.defaultBinaryMessenger.setMockMessageHandler(_FLUTTER_ASSETS_CHANNEL, null);
  }

  Future<Directory> _getExternalStorageDirectory({bool isExternal = false}) async{
    if(isExternal == true) {
      return (await getExternalStorageDirectory())!;
    }
    return getApplicationDocumentsDirectory();
  }

  Future<File?> _downloadFileFromNetwork(String url, String module, {String extension = "zip"}) async {
    if(!(await requestPermissions())){
      logNUI(MODULE, "Please allow storage permission for the app for downloading file");
      return null;
    }

    try{
      var req = await http.Client().get(Uri.parse(url));
      final bytes = req.bodyBytes;
      logNUI(MODULE, "Downloaded the zip file of size : ${bytes.length}");
      final downloadPath = await _getZipFileDownloadName(module, extension: extension);
      var file = File(downloadPath);
      return file.writeAsBytes(bytes);
    }catch(e, s){
      logNUI(MODULE, "Failed to download file from network path: $url with error : $e, stack : $s");
      return null;
    }
  }

  String _getZipFileNameByModule(String module, {String extension: "zip"}){
    return "$module.${extension ?? "zip"}";
  }

  Future<String> _getZipFileDownloadName(String module, {String extension: "zip"}) async{
    final directory = await _getExternalStorageDirectory();
    return "${directory.path}/${_getZipFileNameByModule(module, extension: extension)}";
  }

  Future<File?> _downloadZipFromNetwork(String zipUrl, String module, {String extension = "zip"}) async {
    if(!(await requestPermissions())){
      logNUI(MODULE, "Please allow storage permission for the app for downloading zip");
      return null;
    }

    logNUI(MODULE, "Starting download zip file from url : $zipUrl for module : $module ...");
    final zippedFile = await _downloadFileFromNetwork(zipUrl, module, extension: extension);
    if(zippedFile == null){
      logNUI(MODULE, "Failed to download zip file for module : $module");
      return null;
    }
    else {
      logNUI(MODULE, "Download zip file for module : $module is successful, starting unzip ...");
      final unzipped = await _unarchiveAndSave(zippedFile, module);

      if(unzipped){
        logNUI(MODULE, "Zip file download and unzip completed for module : $module");
        return zippedFile;
      }
      else{
        logNUI(MODULE, "Zip file downloaded but unzip failed, deleting zip of module : $module ...");
        zippedFile.delete();
        logNUI(MODULE, "Zip file deleted for module : $module");
        return null;
      }
    }
  }

  Future<String> _getArchiveDirectoryPath(String module) async{
    final directory = await _getExternalStorageDirectory();
    final directoryPath = "${directory.path}/$module";
    return directoryPath;
  }

  // Unarchive and save the file in Documents directory and save the paths in the array
  Future<bool> _unarchiveAndSave(File zippedFile, String module) async {
    if(!(await requestPermissions())){
      logNUI(MODULE, "Please allow storage permission for the app for unzip");
      return false;
    }

    final directoryPath = await _getArchiveDirectoryPath(module);

    logNUI(MODULE, "Start unzipping zip file into path : $directoryPath");
    String? parentFolder;
    try{
      var bytes = zippedFile.readAsBytesSync();
      var archive = ZipDecoder().decodeBytes(bytes);
      for (var file in archive) {
        logNUI(MODULE, "Original file name: ${file.name}");
        final zipFileName = parentFolder != null ? file.name.replaceFirst("$parentFolder/", "") : file.name;
        var fileName = '$directoryPath/$zipFileName';
        if (file.isFile) {
          logNUI(MODULE, "Inner file in zip : $fileName");
          var outFile = File(fileName);
          outFile = await outFile.create(recursive: true);
          await outFile.writeAsBytes(file.content);
        }
        else{
          //The folder names
          final splittedFileName = file.name.split("/"); //To remove the folder name of the zip file
          splittedFileName.remove("");
          if(splittedFileName.length == 1){
            //The parent folder
            parentFolder = splittedFileName[0];
            logNUI(MODULE, "The parent folder name : $parentFolder");
          }
        }
      }
      logNUI(MODULE, "Unzip zip files successful with ${archive.length} files");
      return true;
    }catch(e){
      logNUI(MODULE, "Unzip files failed with error : $e");
      return false;
    }
  }

}
